<?php

namespace Drupal\webformnavigation;

use Drupal;
use Drupal\webform\Utility\WebformElementHelper;
use Drupal\webform\WebformSubmissionForm;
use Drupal\webform\WebformSubmissionInterface;

/**
 * Defines a helper class for the webform navigation module.
 */
class WebformNavigationHelper {

  /**
   * An array of errors to be logged.
   *
   * @var array
   */
  private static $stashedErrors;

  /**
   * A string for the error log operation
   *
   * @var string
   */
  private static $error_operation = 'navigation error';

  /**
   * Getter for webform submission errors.
   *
   * @param \Drupal\webform\WebformSubmissionInterface $webform_submission
   *   A webform submission entity.
   *
   * @return array
   *   Array of errors per page.
   */
  public static function validateWebformSubmissionValues(WebformSubmissionInterface $webform_submission) {
    $page_errors = [];
    $values = [
      'webform_id' => $webform_submission->getWebform()->id(),
      'data' => $webform_submission->getData(),
    ];
    // Set our errors if they are not NULL.
    $form_values_errors = WebformSubmissionForm::validateFormValues($values);
    if (!empty($form_values_errors)) {
      // Convert the errors to strings.
      WebformElementHelper::convertRenderMarkupToStrings($form_values_errors);
      $page_errors = $form_values_errors;
    }
    // Return either the values errors or an empty array.
    return $page_errors;
  }

  /**
   * Stashes errors to the local variable.
   *
   * @param array $errors
   *   Array of errors to stash.
   */
  public static function stashErrors(array $errors) {
    self::$stashedErrors = $errors;
  }

  /**
   * Getter for stashed errors.
   *
   * @return array
   *   An array of stashed errors.
   */
  public static function getStashedErrors() {
    return empty(self::$stashedErrors) ? [] : self::$stashedErrors;
  }

  /**
   * Get the last submission id.
   *
   * @param string $operation
   *   The operation to target.
   * @param string $sid
   *   The submission ID for the webform.
   *
   * @return object
   *   The last submission log.
   */
  protected static function getLastSubmissionLog($operation, $sid) {
    $query = \Drupal::database()->select('webform_submission_log', 'l');
    $query->condition('operation', $operation);
    $query->condition('sid', $sid);
    $query->fields('l', [
      'lid',
      'sid',
      'data',
    ]);
    $query->orderBy('l.lid', 'DESC');
    $query->range(0, 1);
    $submission_log = $query->execute()->fetch();
    if ($submission_log) {
      $submission_log->data = unserialize($submission_log->data);
    }
    return $submission_log;
  }

  /**
   * Get the last logged errors.
   *
   * @param \Drupal\webform\WebformSubmissionInterface $webform_submission
   *   A webform submission entity.
   *
   * @return array
   *   The logged errors.
   */
  public static function getLastLoggedErrors(WebformSubmissionInterface $webform_submission) {
    // Get outta here if the submission hasn't been saved yet.
    if (empty($webform_submission->id())) {
      return [];
    }
    $error_log = self::getLastSubmissionLog(self::$error_operation, $webform_submission->id());
    // Return data if the $error_log is not NULL.
    return empty($error_log) ? [] : $error_log->data;
  }

  /**
   * Get the last logged errors for a page
   *
   * @param \Drupal\webform\WebformSubmissionInterface $webform_submission
   *   A webform submission entity.
   * @param string $page
   *  The ID for the page we need to retrieve the errors from.
   *
   * @return array
   *   The logged errors.
   */
  public static function getLastSubmissionErrorLogForPage(WebformSubmissionInterface $webform_submission, $page) {
    $page_elements = self::getDecodedElementsForPage($webform_submission, $page);
    $logged_errors = self::getLastLoggedErrors($webform_submission);
    $page_errors = [];
    if (isset($logged_errors[$page])) {
      foreach ($page_elements as $element_key => $element) {
        // Check to see if the element is a composite.
        if (isset($element['#type']) && $element['#type'] == 'webform_custom_composite' && isset($element['#element'])) {
          $composite_element_errors = self::getCompositeElementsErrors($logged_errors[$page], $element, $element_key);
          foreach ($composite_element_errors as $composite_element_error_key => $composite_element_error) {
            $page_errors[$composite_element_error_key] = $composite_element_error;
          }
        }
        elseif (isset($logged_errors[$page][$element_key])) {
          $page_errors[$element_key] = $logged_errors[$page][$element_key];
        }
      }
    }
    return $page_errors;
  }

  /**
   * Get a composite element's errors.
   *
   * @return array
   *   The logged errors.
   */
  public static function getCompositeElementsErrors(array $logged_errors, array $composite_element, $element_id) {
    $composite_errors = [];
    // Parse through each child element.
    foreach ($composite_element['#element'] as $composite_child_key => $composite_child) {
      // See if a child is associated with an error.
      foreach ($logged_errors as $error_key => $error) {
        $parent_string = $element_id . ']';
        $child_string = '[' . $composite_child_key;
        $contains_parent_string = strpos($error_key, $parent_string);
        $contains_child_string = strpos($error_key, $child_string);
        // Set the error if a composite child error exists.
        if ($contains_parent_string !== FALSE && $contains_child_string !== FALSE) {
          $composite_errors[$error_key] = $error;
        }
      }
    }
    return $composite_errors;
  }

  /**
   * Get all the elements for a page.
   *
   * @param \Drupal\webform\WebformSubmissionInterface $webform_submission
   *   A webform submission entity.
   * @param string $page_id
   *   The ID for the page we need to get the elements for.
   *
   * @return array
   *   The page's elements.
   */
  public static function getDecodedElementsForPage(WebformSubmissionInterface $webform_submission, $page_id) {
    $all_elements = $webform_submission->getWebform()->getElementsDecoded();
    return $all_elements[$page_id];
  }

  /**
   * Logs webform submission errors.
   *
   * @param \Drupal\webform\WebformSubmissionInterface $webform_submission
   *   A webform submission entity.
   * @param array $errors
   *   Array of errors to log.
   */
  public static function logErrors(WebformSubmissionInterface $webform_submission, array $errors) {
    self::logSubmissionData($webform_submission, self::$error_operation, $errors);
  }

  /**
   * Updates the webform submission errors.
   *
   * @param \Drupal\webform\WebformSubmissionInterface $webform_submission
   *   A webform submission entity.
   * @param array $errors
   *   Array of errors to log.
   */
  public static function updateErrorLogs(WebformSubmissionInterface $webform_submission, array $errors) {
    // Get any previously logged errors if the submission has been saved.
    $current_errors = !empty($webform_submission->id()) ? self::getLastLoggedErrors($webform_submission) : [];
    $page = self::getCurrentPage($webform_submission);
    $page_elements = self::getDecodedElementsForPage($webform_submission, $page);
    // Record the page for historic tracking.
    $current_errors[$page] = [];
    // Remove the previous errors for the current page.
    if (isset($current_errors[$page])) {
      foreach ($page_elements as $element_key => $element) {
        // Remove the previous composite child element's errors.
        if (isset($element['#type']) && $element['#type'] == 'webform_custom_composite' && isset($element['#element'])) {
          $composite_element_errors = self::getCompositeElementsErrors($current_errors[$page], $element, $element_key);
          // Remove the composite errors if they are set.
          foreach (array_keys($composite_element_errors) as $composite_element_error_key) {
            if (isset($current_errors[$page][$composite_element_error_key])) {
              unset($current_errors[$page][$composite_element_error_key]);
            }
          }
        }
        // Remove errors on basic page elements if they are set.
        elseif (isset($current_errors[$page][$element_key])) {
          unset($current_errors[$page][$element_key]);
        }
      }
    }
    // Set the supplied errors.
    foreach ($errors as $key => $error) {
      $current_errors[$page][$key] = $error;
    }
    // Update the error log.
    self::logErrors($webform_submission, $current_errors);
  }

  /**
   * Re-writes the webform submission error logs.
   *
   * @param \Drupal\webform\WebformSubmissionInterface $webform_submission
   *   A webform submission entity.
   * @param array $errors
   *   Array of errors to log.
   */
  public static function logSubmissionErrors(WebformSubmissionInterface $webform_submission, array $errors) {
    // Get any previously logged errors if the submission has been saved.
    $paged_errors = [];
    $pages = $webform_submission->getWebform()->getPages();
    // Remove the previous errors for the current page.
    foreach ($pages as $page_key => $page) {
      $page_elements = self::getDecodedElementsForPage($webform_submission, $page_key);
      $paged_errors[$page_key] = [];
      foreach ($page_elements as $element_key => $element) {
        // Check to see if the element is a composite.
        if (isset($element['#type']) && $element['#type'] == 'webform_custom_composite' && isset($element['#element'])) {
          $composite_element_errors = self::getCompositeElementsErrors($errors, $element, $element_key);
          foreach ($composite_element_errors as $composite_element_error_key => $composite_element_error) {
            $paged_errors[$page_key][$composite_element_error_key] = $composite_element_error;
          }
        }
        elseif (isset($errors[$element_key])) {
          $paged_errors[$page_key][$element_key] = $errors[$element_key];
        }
      }
    }
    // Update the error log.
    self::logErrors($webform_submission, $paged_errors);
  }

  /**
   * Updates the webform submission errors with our stashed error array.
   *
   * @param \Drupal\webform\WebformSubmissionInterface $webform_submission
   *   A webform submission entity.
   */
  public static function logStashedErrors(WebformSubmissionInterface $webform_submission) {
    // Get any previously logged errors if the submission has been saved.
    $current_errors = !empty($webform_submission->id()) ? self::getLastLoggedErrors($webform_submission) : [];
    $stashed_errors = self::$stashedErrors;
    // Return if there are no stashed errors.
    if (empty($stashed_errors)) {
      return;
    }
    // Set the supplied errors.
    foreach ($stashed_errors as $page => $errors) {
      $current_errors[$page] = $errors;
    }
    // Update the error log.
    self::logErrors($webform_submission, $current_errors);
  }

  /**
   * Gets the current submission page.
   *
   * @param \Drupal\webform\WebformSubmissionInterface $webform_submission
   *   A webform submission entity.
   *
   * @return string
   *   The current submission page ID.
   */
  public static function getCurrentPage(WebformSubmissionInterface $webform_submission) {
    $pages = $webform_submission->getWebform()->getPages();
    return empty($webform_submission->getCurrentPage()) ? array_keys($pages)[0] : $webform_submission->getCurrentPage();
  }

  /**
   * Logs submission data.
   *
   * @param \Drupal\webform\WebformSubmissionInterface $webform_submission
   *   A webform submission entity.
   * @param string $operation
   *   The name of the operation for the log.
   * @param array $data
   *   Array of data to log.
   */
  private static function logSubmissionData(WebformSubmissionInterface $webform_submission, $operation, array $data) {
    $message = 'Webform navigation updated for submission @webform_submission_sid';
    $context = [
      'webform_submission' => $webform_submission,
      'operation' => $operation,
      'data' => $data,
      '@webform_submission_sid' => $webform_submission->id(),
    ];
    \Drupal::logger('webform_submission')->notice($message, $context);
  }

}
